package provide Megawidget 0.1

#########################################################################
#                                                                       #
# Script:  megawidget.tcl                                               #
#                                                                       #
# Author:  Dave Freese, dave@darkmatterfactory.com                      #
#                                                                       #
# Provides:  common support code for                                    #
#            calendar                                                   #
#            dateentry &                                                #
#            notebook                                                   #
#                                                                       #
# Version:  v0.1.0                                                      #
# --------------------------------------------------------------------- #
#                                                                       #
#                                                                       #
# Version:  0.1.0                                                       #
# Date:    9 October, 2001                                              #
# Author:  Dave Freese, dave@darkmatterfactory.com                      #
# Change:  Original released version.                                   #
#                                                                       #
# Version:                                                              #
# Date:                                                                 #
# Author:                                                               #
# Change:                                                               #
#                                                                       #
#########################################################################

###############################################################################
#    megawidget.tcl - tcl module to provide common code for megawidget        #
#                     creation                                                #
#    Copyright (C) 2001 Dave Freese                                           #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program; if not, write to the Free Software              #
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  #
#                                                                             #
###############################################################################


package require Tk 8.0

namespace eval ::megawidget {
  namespace export megawidget
}


# ::megawidget::Canonize --
#
#    takes a (possibly abbreviated) option or command name and either 
#    returns the canonical name or an error
#
# Arguments:
#
#    w        widget pathname
#    object   type of object to canonize; must be one of "command",
#             "option", "scan command" or "list command"
#    opt      the option (or command) to be canonized
#
# Returns:
#
#    Returns either the canonical form of an option or command,
#    or raises an error if the option or command is unknown or
#    ambiguous.
#
# Canonize expects that all procedure and variables associated with
# the widget 'w' will be lowercase and that the class associated with
# 'w' will be written with the first character upper case.

proc ::megawidget::Canonize {w object opt} {
  set wclass [string tolower [winfo class ${w}]]
  upvar  ::${wclass}::widgetCommands  widgetCommands
  upvar  ::${wclass}::widgetOptions widgetOptions
  upvar  ::${wclass}::${w}::options options

  switch $object {
    command {
      if {[lsearch -exact $widgetCommands $opt] >= 0} {
        return $opt
      }
      # stuff command names in temporary array 
      set list $widgetCommands
      foreach element $list {
        set tmp($element) ""
      }
      set matches [array names tmp ${opt}*]
    }

    option {
      if {[info exists widgetOptions($opt)] \
        && [llength $widgetOptions($opt)] == 2} {
        return $opt
      }
      set list [array names widgetOptions]
      set matches [array names widgetOptions ${opt}*]
    }
  }

  if {[llength $matches] == 0} {
    set choices [HumanizeList $list]
    error "unknown $object \"$opt\"; must be one of $choices"
  } elseif {[llength $matches] == 1} {
    set opt $matches
    # deal with option aliases
    if {$option == "option"} {
      if {[llength $widgetOptions($opt)] == 1} {
        set opt $widgetOptions($opt)
      }
    }
    return $opt
  } else {
    set choices [HumanizeList $list]
    error "ambiguous $object \"$opt\"; must be one of $choices"
  }
}

# ::megawidget::HumanizeList --
#
#    Returns a human-readable form of a list by separating items
#    by columns, but separating the last two elements with "or"
#    (eg: foo, bar or baz)
#
# Arguments:
#
#    list    a valid tcl list
#
# Results:
#
#    A string which as all of the elements joined with ", " or 
#    the word " or "

proc ::megawidget::HumanizeList {list} {

  if {[llength $list] == 1} {
    return [lindex $list 0]
  } else {
    set list [lsort $list]
    set secondToLast [expr {[llength $list] -2}]
    set most [lrange $list 0 $secondToLast]
    set last [lindex $list end]

    return "[join $most {, }] or $last"
  }
}

# ::megawidget::DestroyHandler {w} --
# 
#    Cleans up after an associated widget is destroyed
#
# Arguments:
#
#    w    widget pathname
#
# Results:
#
#    The namespace that was created for the widget is deleted,
#    and the widget proc is removed.

proc ::megawidget::DestroyHandler {w} {
  set wclass [winfo class ${w}]
  upvar ::${wclass}::${w}::widgets  widgets
  upvar ::${wclass}::${w}::options  options

  namespace delete ::${wclass}::$w
  rename $w {}

  return ""
}

